
//
// AttachmentHelper.cs
//
// Author:
//    Tomas Restrepo (tomasr@mvps.org)
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

using Microsoft.XLANGs.BaseTypes;
using Microsoft.XLANGs.Core;

using Winterdom.BizTalk.Samples.BinaryHelper;

/// <summary>
/// Helper class to attach arbitrary binary 
/// parts to a multi-part message from 
/// an orchestration
/// </summary>
public sealed class AttachmentHelper
{
   /// <summary>
   /// Attach a file from the filesystem
   /// as a binary part into the message
   /// </summary>
   /// <param name="message">Orchestration message to attach to</param>
   /// <param name="filename">Path to the file to attach. 
   /// Make sure biztalk has access to it</param>
   public static void AttachFile(XLANGMessage message, string filename)
   {
      if ( message == null )
         throw new ArgumentNullException("message");
      if ( filename == null )
         throw new ArgumentNullException("filename");

      string partName = Path.GetFileName(filename);
      Stream stream = File.Open(filename, FileMode.Open);

      AttachStream(message, partName, stream);
   }

   /// <summary>
   /// Attach an arbitrary stream 
   /// as a binary part into the message
   /// </summary>
   /// <param name="message">Orchestration message to attach to</param>
   /// <param name="filename">Stream to attach</param>
   /// <remarks>
   /// Make sure the stream is position correctly (at position 0, normally).
   /// Also, this method will take ownership of the stream, so don't touch it
   /// afterwards.
   /// </remarks>
   public static void AttachStream(XLANGMessage message, string partName, Stream stream)
   {
      if ( message == null )
         throw new ArgumentNullException("message");
      if ( partName == null )
         throw new ArgumentNullException("partName");
      if ( stream == null )
         throw new ArgumentNullException("stream");

      IStreamFactory factory = new BinaryStreamFactory(stream);
      message.AddPart(factory, partName);

      //
      // set the Filename property so that it 
      // gets correctly recognized in the mail
      // reader app when it is received as an attachment
      //
      XLANGPart part = message[partName];
      part.SetPartProperty(typeof(MIME.FileName), partName);
   }

   public static void AttachByteArray(XLANGMessage message, string partName, byte[] buffer)
   {
      if ( message == null )
         throw new ArgumentNullException("message");
      if ( partName == null )
         throw new ArgumentNullException("partName");
      if ( buffer == null )
         throw new ArgumentNullException("buffer");

      MemoryStream stream = new MemoryStream(buffer);
      AttachStream(message, partName, stream);
   }

} // class AttachmentHelper

